#import "AppDelegate.h"

#import "LiveCodeView.h"

@implementation AppDelegate

@synthesize window = _window;

////////////////////////////////////////

- (id)init
{
    self = [super init];
    if (self == nil)
        return nil;
    
    m_livecode_view = nil;
    m_position_label = nil;

    return self;
}

- (void)dealloc
{
    [m_livecode_view release];
    [m_position_label release];
    [super dealloc];
}

////////////////////////////////////////

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Get the screen bounds so we can position our objects.
    CGRect t_screen_bounds;
    t_screen_bounds = [[UIScreen mainScreen] bounds];
    
    // Create the main window of the application.
    self.window = [[UIWindow alloc] initWithFrame: t_screen_bounds];
    [self.window setBackgroundColor: [UIColor redColor]];
    
    // Create a full screen view in which we will place our controls.
    UIView *t_container_view;
    t_container_view = [[[UIView alloc] initWithFrame: t_screen_bounds] autorelease];
    
    // Fetch the LiveCodeView for our embedded application.
    m_livecode_view = LiveCodeGetView();
    
    // Set the bounds of our embedded view. Note that (for the moment) we must
    // use 'setViewBounds' rather than the more natural 'setBounds' or 'setFrame'
    // messages.
    [m_livecode_view setViewBounds: CGRectMake(t_screen_bounds . size . width / 2.0f - 160.0f, 20.0f, 320.0f, 320.0f)];
    
    // Map the message 'movingLocationChanged' to a selector on this object.
    [m_livecode_view setDelegate: self];
    [m_livecode_view mapMessage: "imageLocationChanged" withSignature: "vii" toSelector: @selector(imageLocationChangedToX:Y:)];
    
    // Create the start and stop buttons and the display label.
    m_position_label = [[UILabel alloc] initWithFrame: CGRectMake(t_screen_bounds . size . width / 2.0f - 160.0f, 20.0f + 320.0f + 8.0f, 320.0f, 20.0f)];
    [m_position_label setTextAlignment: UITextAlignmentCenter];
    [m_position_label setText: @"not moving"];
    UIButton *t_start_button, *t_stop_button;
    t_start_button = [UIButton buttonWithType: UIButtonTypeRoundedRect];
    [t_start_button setFrame: CGRectMake(t_screen_bounds . size . width / 2.0f - 160.0f, 20.0f + 320.0f + 8.0f + 20.0f + 8.0f, 156.0f, 40.0f)];
    [t_start_button setTitle: @"Start" forState: UIControlStateNormal];
    [t_start_button addTarget: self action: @selector(startMoving) forControlEvents: UIControlEventTouchUpInside];
    t_stop_button = [UIButton buttonWithType: UIButtonTypeRoundedRect];
    [t_stop_button setFrame: CGRectMake(t_screen_bounds . size . width / 2.0f + 4.0f, 20.0f + 320.0f + 8.0f + 20.0f + 8.0f, 156.0f, 40.0f)];
    [t_stop_button setTitle: @"Stop" forState: UIControlStateNormal];
    [t_stop_button addTarget: self action: @selector(stopMoving) forControlEvents: UIControlEventTouchUpInside];
    
    // Add the views to the container.
    [t_container_view addSubview: m_livecode_view];
    [t_container_view addSubview: m_position_label];
    [t_container_view addSubview: t_start_button];
    [t_container_view addSubview: t_stop_button];
    
    // Create a view controller.
    UIViewController *t_view_controller;
    t_view_controller = [[[UIViewController alloc] init] autorelease];
    
    // Set the view of the controller to the container view.
    [t_view_controller setView: t_container_view];
    
    // Set the window's controller to be our fresh new one and release it.
    [self.window setRootViewController: t_view_controller];
    [t_view_controller release];
    
    // Tell the engine to startup.
    [m_livecode_view startup];
    
    // Present the window.
    [self.window makeKeyAndVisible];
    
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Tell the engine to shutdown.
    [m_livecode_view shutdown];
}

////////////////////////////////////////

// This method is invoked when the start button is pressed.
- (void)startMoving
{
    [m_livecode_view post: @"startMoving"];
}

// This method is invoked when the stop button is pressed.
- (void)stopMoving
{
    [m_livecode_view post: @"stopMoving"];
}

// This method is invoked by the engine when the 'imageLocationChanged' message
// passes through the message path.
- (void)imageLocationChangedToX:(int)p_x Y:(int)p_y
{
    [m_position_label setText: [NSString stringWithFormat:@"moved to (%d, %d)", p_x, p_y]];
}

////////////////////////////////////////

@end
